//importing the required methods
import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
} from 'd3';

//defining the scatterplot function
export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let symbolValue;
  let margin;
  let size;

  const sca = (selection) => {
    //defining the x-axis range
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);
    //defining the y-axis range
    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);
    
    //defining the range of the symbols
    const symbolScale = scaleOrdinal()
      .domain(data.map(symbolValue))
      .range(symbols);
    
    //defining the symbolgenerator function to generate the symbols of different iris species
    const symbolGenerator = symbol().size(size);
    
    //loading the iris data coordinates
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      pathD: symbolGenerator.type(symbolScale(symbolValue(d)))()
    }));
    
    //adding the path to place the iris data
    selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('d', (d) => d.pathD)
      .attr('transform', (d) => `translate(${d.x},${d.y})`);

    //appending the group for the x-axis
    selection
      .append('g')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));
    
    //appending a group for the y-axis
    selection
      .append('g')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
  };
  
  //defining the function for width of the scatter plot
  sca.width = function (_) {
    return arguments.length ? ((width = +_), sca) : width;
  };
  
  //defining the function for height of the scatter plot
  sca.height = function (_) {
    return arguments.length ? ((height = +_), sca) : height;
  };
  
  //defining the function for loading the data
  sca.data = function (_) {
    return arguments.length ? ((data = _), sca) : data;
  };
  
  //defining the function for xvalue of the iris data
  sca.xValue = function (_) {
    return arguments.length ? ((xValue = _), sca) : xValue;
  };
   
  //defining the function for yvalue of the iris data
  sca.yValue = function (_) {
    return arguments.length ? ((yValue = _), sca) : yValue;
  };
  
  //defining the function for symbolvalue of the iris data
  sca.symbolValue = function (_) {
    return arguments.length
      ? ((symbolValue = _), sca)
      : symbolValue;
  };
  
  //defining the function for margin of the scatter plot
  sca.margin = function (_) {
    return arguments.length ? ((margin = _), sca) : margin;
  };
  
  //defining the function for size of the scatter plot
  sca.size = function (_) {
    return arguments.length ? ((size = +_), sca) : size;
  };

  return sca;
};
